import 'dart:io';

import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:jiffy/jiffy.dart';
import 'package:provider/provider.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:flutter_localizations/flutter_localizations.dart';

import 'app.dart';
import 'model/accountModel.dart';
import 'model/chatList/chatListProps.dart';
import 'model/chatList/chatProps.dart';
import 'model/files/filesProps.dart';
import 'model/message/messageProps.dart';
import 'model/timetable/timetableProps.dart';
import 'storage/base/settingsProvider.dart';
import 'theming/darkAppTheme.dart';
import 'theming/lightAppTheme.dart';
import 'view/login/login.dart';
import 'widget/errorView.dart';

Future<void> main() async {
  WidgetsFlutterBinding.ensureInitialized();

  ByteData data = await PlatformAssetBundle().load('assets/ca/lets-encrypt-r3.pem');
  SecurityContext.defaultContext.setTrustedCertificatesBytes(data.buffer.asUint8List());

  ErrorWidget.builder = (error) {
    return ErrorView(icon: Icons.phonelink_erase_rounded, text: error.toString());
  };

  runApp(
      MultiProvider(
        providers: [
          ChangeNotifierProvider(create: (context) => SettingsProvider()),
          ChangeNotifierProvider(create: (context) => AccountModel()),

          ChangeNotifierProvider(create: (context) => TimetableProps()),
          ChangeNotifierProvider(create: (context) => ChatListProps()),
          ChangeNotifierProvider(create: (context) => ChatProps()),
          ChangeNotifierProvider(create: (context) => FilesProps()),

          ChangeNotifierProvider(create: (context) => MessageProps()),
        ],
        child: const Main(),
      )
  );
}

class Main extends StatefulWidget {
  const Main({Key? key}) : super(key: key);

  @override
  State<Main> createState() => _MainState();
}

class _MainState extends State<Main> {

  final Future<SharedPreferences> _storage = SharedPreferences.getInstance();

  @override
  void initState() {
    super.initState();
    Jiffy.setLocale("de");
    _storage.then((SharedPreferences preferences) => preferences.getBool("loggedIn") ?? false).then((value) => {
      if(value) {
        Provider.of<AccountModel>(context, listen: false).login()
      } else {
        Provider.of<AccountModel>(context, listen: false).logout()
      }
    });

  }

  @override
  Widget build(BuildContext context) {
    SystemChrome.setPreferredOrientations([DeviceOrientation.portraitUp]);

    return Directionality(
      textDirection: TextDirection.ltr,
      child: Consumer<SettingsProvider>(
        builder: (context, settings, child) {
          return MaterialApp(
              debugShowCheckedModeBanner: false,
              localizationsDelegates: const [
                ...GlobalMaterialLocalizations.delegates,
                GlobalWidgetsLocalizations.delegate,
              ],
              supportedLocales: const [
                Locale('de'),
                Locale('en'),
              ],
              locale: const Locale('de'),


              title: 'Marianum Fulda',

              themeMode: settings.val().appTheme,
              theme: LightAppTheme.theme,
              darkTheme: DarkAppTheme.theme,


              home: FutureBuilder<SharedPreferences>(
                future: _storage,
                builder: (BuildContext context, AsyncSnapshot<SharedPreferences> snapshot) {

                  if(snapshot.hasData) {
                    return Consumer<AccountModel>(
                      builder: (context, accountModel, child) {
                        return accountModel.isLoggedIn ? const App() : const Login();
                      },
                    );
                  } else {
                    return const Center(child: CircularProgressIndicator());
                  }
                },
              )
          );
        },
      ),
    );
  }
}
