import 'package:flutter/material.dart';
import 'package:provider/provider.dart';

import '../../../../storage/base/settingsProvider.dart';
import '../../../../widget/confirmDialog.dart';

class GradeAverage extends StatefulWidget {
  const GradeAverage({Key? key}) : super(key: key);

  @override
  State<GradeAverage> createState() => _GradeAverageState();
}

class _GradeAverageState extends State<GradeAverage> {
  double average = 0;
  bool gradeSystem = true;
  List<int> grades = List.empty(growable: true);

  late SettingsProvider settings = Provider.of<SettingsProvider>(context, listen: false);
  
  void _persistGrades() {
    settings.val(write: true).gradeAveragesSettings.inputs = grades;
  }

  String getGradeDisplay(int grade) {
    if(gradeSystem) {
      return "Note $grade";
    } else {
      return "$grade Punkt${grade > 1 ? "e" : ""}";
    }
  }

  @override
  void initState() {
    super.initState();
    
    grades = settings.val().gradeAveragesSettings.inputs;
    gradeSystem = settings.val().gradeAveragesSettings.useGradeSystem;
  }

  @override
  Widget build(BuildContext context) {
    if(grades.isNotEmpty) {
       average = grades.reduce((a, b) => a + b) / grades.length;
    } else {
      average = 0;
    }

    return Scaffold(
      appBar: AppBar(
        title: const Text("Notendurschnittsrechner"),
        actions: [
          IconButton(onPressed: () {
            showDialog(
              context: context,
              builder: (context) => ConfirmDialog(
                title: "Zurücksetzen?",
                content: "Alle Einträge werden entfernt.",
                confirmButton: "Zurücksetzen",
                onConfirm: () {
                  grades.clear();
                  setState(() {});
                },
              ),
            );
          }, icon: const Icon(Icons.delete_forever)),
          PopupMenuButton<bool>(
            enableFeedback: true,
            initialValue: gradeSystem,
            icon: const Icon(Icons.more_horiz),
            itemBuilder: (context) => [true, false].map((e) => PopupMenuItem<bool>(
              value: e,
              enabled: e != gradeSystem,
              child: Row(
                children: [
                  Text(e ? "Notensystem" : "Punktesystem"),
                ],
              ),
            )).toList(),

            onSelected: (e) {
              void switchSystem() => setState(() {
                grades.clear();
                gradeSystem = e;
                settings.val(write: true).gradeAveragesSettings.useGradeSystem = e;
                _persistGrades();
              });

              if(grades.isNotEmpty) {
                showDialog(
                  context: context,
                  builder: (context) => ConfirmDialog(
                    title: "Notensystem wechseln",
                    content: "Beim wechsel des Notensystems werden alle Einträge zurückgesetzt.",
                    confirmButton: "Fortfahren",
                    onConfirm: () => switchSystem(),
                  ),
                );
              } else {
                switchSystem();
              }


            },
          ),
        ],
      ),
      body: Column(
        crossAxisAlignment: CrossAxisAlignment.center,
        mainAxisAlignment: MainAxisAlignment.center,
        children: [
          const SizedBox(height: 30),
          Text(average.toStringAsFixed(2), style: const TextStyle(fontSize: 30, fontWeight: FontWeight.bold)),
          const SizedBox(height: 10),
          const Divider(),
          const SizedBox(height: 10),
          Text(gradeSystem ? "Wähle unten die Anzahl deiner jewiligen Noten aus" : "Wähle unten die Anzahl deiner jeweiligen Punkte aus"),
          const SizedBox(height: 10),
          Expanded(
            child: ListView.builder(
              itemBuilder: (context, index) {
                var grade = gradeSystem ? index + 1 : 14 - index + 1;
                bool isThis(int e) => e == grade;
                return Material(
                  child: ListTile(
                    tileColor: grade.isEven ? Colors.transparent : Colors.transparent.withAlpha(50),
                    title: Center(
                      child: Row(
                        mainAxisAlignment: MainAxisAlignment.center,
                        children: [
                          Text(getGradeDisplay(grade)),
                          const SizedBox(width: 30),
                          IconButton(
                            onPressed: () {
                              setState(() {
                                if(!grades.any(isThis)) return;
                                grades.removeAt(grades.indexWhere(isThis));
                                _persistGrades();
                              });
                            },
                            icon: const Icon(Icons.remove),
                            color: Theme.of(context).colorScheme.onSurface,
                          ),
                          Text("${grades.where(isThis).length}", style: const TextStyle(fontSize: 15, fontWeight: FontWeight.bold)),
                          IconButton(
                            onPressed: () {
                              setState(() {
                                grades.add(grade);
                                _persistGrades();
                              });
                            },
                            icon: const Icon(Icons.add),
                            color: Theme.of(context).colorScheme.onSurface,
                          ),
                        ],
                      ),
                    ),
                    trailing: Visibility(
                      maintainState: true,
                      maintainAnimation: true,
                      maintainSize: true,
                      visible: grades.any(isThis),
                      child: IconButton(
                        icon: const Icon(Icons.delete),
                        onPressed: () {
                          setState(() {
                            grades.removeWhere(isThis);
                            _persistGrades();
                          });
                        },
                      ),
                    ),
                  ),
                );
              },
              itemCount: gradeSystem ? 6 : 15,
            ),
          ),
        ],
      ),
    );
  }
}
